


// Includes definitions relevant to the peripherals embedded in the digital signal controller.
#include "F2806x_Device.h"

#include "constants.h"

#include "hardware_interface.h"




// Definition of external functions. The bodies of the functions are in another file
extern int Initialize_the_Clocks(void);
extern int Initialize_the_GPIO(void);
extern int Initialize_the_Timer0(int);
extern int Initialize_the_ADC(void);
extern int Initialize_the_PWM(void);
extern int Enable_the_Peripherals(void);
extern int Manage_LEDs(int, int);
extern int Show_Character_on_Digit(char,int);
extern int Sample_the_Analog_Voltage(void);
extern int Sample_the_Temperature(void);
extern int Set_PWM_Duty_Cycles(int, int, int);

extern int Enable_the_Interrupts(void);


extern const float Sine_table[128];
extern float Interpolate_Sine_Table(float, float*);

int Number_to_Char_Converter(int,char*);

// Global variables are defined outside any function.
int Contatore=0;

int Global_Sample;


#define CONTATORE1_INIZIALE 1000
#define CONTATORE2_INIZIALE 2000


// The program execution by default starts from the function called "main"
int main(void){

    int Rosso=0;
    int Contatore1=0;
    int Contatore2=0;
    int Contatore3=-100;

    int Digit=1;
    int Esito_Display;
    int Number_to_be_Converted;

    int Red_Duty;
    int Blue_Duty;
    int Green_Duty;

    char Four_Digits[4]={'2','1','0','1'};

    char Char_to_Display;


    float Angle;
    float Sine;

    Enable_the_Peripherals();
    Initialize_the_Timer0(10);
    Initialize_the_Clocks();
    Initialize_the_GPIO();
    Initialize_the_ADC();
    Initialize_the_PWM();

    Enable_the_Interrupts();

    ATTIVA_IL_TIMER0;

    while (1) {


        while(Contatore1>0){
            Contatore1--;
            while(Contatore2>0){
                Contatore2--;
            }
            Contatore2=CONTATORE2_INIZIALE;


            if(++Digit>4){
                Digit=1;
            }

            Char_to_Display=Four_Digits[Digit-1];

            Esito_Display=Show_Character_on_Digit(Char_to_Display,Digit);

            if(Esito_Display==CORRECT){
                ACCENDI_VERDE;
                SPEGNI_ROSSO;
            }
            else{
                ACCENDI_ROSSO;
                SPEGNI_VERDE;
            }



        }

        Contatore1=CONTATORE1_INIZIALE;


        if(Contatore3++>100){
            Contatore3=-100;
        }


        Number_to_be_Converted=Contatore3;

        Angle=(((float) Global_Sample/MAXIMUM_ADC_RESULT)-0.5)*PI;

        Interpolate_Sine_Table(Angle,&Sine);

        Red_Duty=(int)((Sine+1)*PWM_TIME_BASE_PERIOD_2);
        Blue_Duty=0;
        Green_Duty=PWM_TIME_BASE_PERIOD-Red_Duty;

        Set_PWM_Duty_Cycles(Red_Duty,Blue_Duty,Green_Duty);




/*************************************************************************************************/
/* Per far funzionare correttamente il programma bisogna impostare correttamente                 */
/*                                                                                               */
/*                                 MAXIMUM_ADC_RESULT                                            */
/*                                                                                               */
/* nel file hardware_interface.h                                                                 */
/*                                                                                               */
/* Per fare ci bisogna eseguire il programma mostrando sul display la variabile Global_Sample,  */
/* agire sul potenzione e prendere nota del massimo valore visualizzato, aggiunge quindi 5-10 e  */
/* assegnare il valore ottenuto a MAXIMUM_ADC_RESULT.                                            */
/* Poi si pu visualizzare sul display Red_Duty o altro.                                         */
/*                                                                                               */
/*************************************************************************************************/

        Number_to_Char_Converter(Red_Duty,Four_Digits);


        if(Rosso==0){
            Manage_LEDs(TURN_ON,RED);
            Manage_LEDs(TURN_OFF,BLUE);
            Rosso=1;
        }
        else{
            Manage_LEDs(TURN_OFF,RED);
            Manage_LEDs(TURN_ON,BLUE);
            Rosso=0;
        }






    }
}




interrupt void Timer0_isr(){

//    Global_Sample=Sample_the_Analog_Voltage();


    if(BOTTONE_PREMUTO){
//        ACCENDI_ROSSO;
        ACCENDI_BLU;
    }
    else{
//        SPEGNI_ROSSO;
        SPEGNI_BLU;
    }

    RESETTA_IL_TIMER0;
    PieCtrlRegs.PIEACK.bit.ACK1=1;

}


interrupt void ADC_isr(){

    Global_Sample=AdcResult.ADCRESULT3;
    AdcRegs.ADCINTFLGCLR.bit.ADCINT1=1;
    PieCtrlRegs.PIEACK.bit.ACK1=1;

}





int Number_to_Char_Converter(int Number,char* Char_Vector){
    int Result=CORRECT;
    int Negative=0;

    int Thousands=0;
    int Hundreds=0;
    int Tens=0;
    int Units=0;


    if((Number<=9999) & (Number>=-999)){
        if(Number<0){
            Negative=1;
            Number=-Number;
        }

        Thousands=Number/1000;
        Number=Number-Thousands*1000;

        Hundreds=Number/100;
        Number=Number-Hundreds*100;

        Tens=Number/10;
        Number=Number-Tens*10;

        Units=Number;

        if(Negative==1){
            Char_Vector[0]='-';
        }
        else{
            Char_Vector[0]='0'+Thousands;
        }

        Char_Vector[1]='0'+Hundreds;
        Char_Vector[2]='0'+Tens;
        Char_Vector[3]='0'+Units;


    }
    else{
        Result=ERROR;
        Char_Vector[0]='E';
        Char_Vector[1]='r';
        Char_Vector[2]='r';
        Char_Vector[3]=' ';
    }


    return Result;
}

